/****************************************************************************************
 * 
 * RxTester.c - Source file for RxTester application - Receive tester for S509 board.
 *				Windows NT 3.31/4.0 Links with papi32mc.lib to .exe
 * 
 * Copyright(c) Sangoma Technologies Inc. 1997
 *
 * Author:	Anton Belov
 * 
 * $Log:   C:/anton/pos/rxtester/pvcs/rxtester.c_v  $
 * 
 *    Rev 1.2   06 Mar 1997 12:11:16   Anton
 * DoPosSetup() call added to cancel the filtering
 * 
 *    Rev 1.1   25 Feb 1997 16:43:28   Anton
 * Do not perform RESET_POS and CONFIGURE if we are not the first application
 * that registers with the board
 * 
 *    Rev 1.0   25 Feb 1997 15:47:24   Anton
 * Initial revision.
 * 
 */

//
//***********************************   Includes   *************************************
//
#include <windows.h>				// windows stuff
#include <windowsx.h>				// GET_WM_COMMAND_ID macro
#include <stdio.h>					// 
#include <stdlib.h>					// sprintf ...
#include "..\papi32mc\papi32mc.h"	// PAPI... types, constants and prototypes
#include "resource.h"				// our resources

//
//***********************************   Constants   *************************************
//
#define	POLL_TIMER			500		// time between two polls - on each poll the board
									// checked if there is data available. Value in ms
char *szClassName = "RXTEST";		// application class name - for registering

#define CARD_NUMBER			0		// s509 card number, as configured

//
//********************************   Global variables   *********************************
//
HINSTANCE	ghinstance;				// application instance
HWND		exehwn;					// main window

BOOL		Receiving = FALSE;		// TRUE if we are currently in Receiving state
BYTE		POSPort = 1;			// POS port number to receive stuff

CBLOCK		cBlock;					// Control block to talk to the board

//
//******************************   Function prototypes   ********************************
//

// Main window function - initializes stuff, and waits inside the message loop
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int cmdShow);
// Callback funciton for the main dialog box
LRESULT CALLBACK MainWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam );
// Updates the states of controls of main dialog box
static void UpdateMe(HWND hWnd);
// Put the message into the list box
static void SayMessage(HWND hWnd, char* message);

// ----------------------------------------------------------------------------------
// These are the "real" board functions - as opposite to the interface functions above

// Performs "warm reset" of the board - returns TRUE is the code seems to be loaded
static BOOLEAN CheckIfCardLoaded(void);
// Sets the cofiguration of the board
static BOOLEAN SetConfiguration(void);
// EnablePos() - enables POS port and starting to listen
static BOOLEAN EnablePos(void);
// DisablePos() - disables POS port
static BOOLEAN DisablePos(void);
// Checks if there is data available for receving on the board
static BOOLEAN IsDataAvailable(void);
// Read the data from the board
static BOOLEAN DoReceive(void);
// Performs POS_SETUP command. This will work correctly only for IBM4680 and NCR 2126/2127.
// For NCR 1255 it will return RCODE_INV_PARAM(0x81) - but we'll still have the default
// filtering setting
static BOOLEAN DoPosSetup(void);

//
// ----------------------------------------------------------------------------------


//
//******************************   Implementations   ********************************
//

//
// WinMain
//
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int cmdShow)
{
	MSG				msg;
	BOOL			fRetVal;
	WNDCLASS		wndclass;			// Structure used to register Windows class.
	UINT			timerId;			// timer indetifier
	PAPI_STATUS		Status;				// To hold the status

	ghinstance = hInstance;

	if (hPrevInstance != NULL) return(0);

	wndclass.style         = CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc   = MainWndProc;
	wndclass.cbClsExtra    = 0;
	wndclass.cbWndExtra    = DLGWINDOWEXTRA;
	wndclass.hInstance     = ghinstance;
	wndclass.hIcon         = LoadIcon(ghinstance, "RXTEST");
	wndclass.hCursor       = LoadCursor( NULL, IDC_ARROW );
	wndclass.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
	wndclass.lpszMenuName  = NULL;
	wndclass.lpszClassName = szClassName;

	if (!RegisterClass(&wndclass)) {
		MessageBox(HWND_DESKTOP, "Couldn't register window class.", "RxTester - Fatal Error", MB_ICONSTOP | MB_OK );
		return(0);
	}

	//
	//------------ s509 and PAPI initialization part ---------------
	//
	// Perform PAPI setup call
	if (PAPISetup() == PAPI_STATUS_FAILURE) {
		MessageBox(HWND_DESKTOP, "Couldn't open POSDLA device driver. Please type \"NET START POSDLA\" in the command promt and try again.", "RxTester - Fatal Error", MB_ICONSTOP | MB_OK );
		return(0);
	}
	// Register the card
	Status = PAPIRegisterCard(CARD_NUMBER);
	if ( PAPI_ERROR(Status) ) {
		PAPICleanup();
		MessageBox(HWND_DESKTOP, "Couldn't register POSDLA card. Try to restart the driver.", "RxTester - Fatal Error", MB_ICONSTOP | MB_OK );
		return(0);
	}
	if (Status == PAPI_STATUS_SUCCESS) { // We're the first application using the card	
										 // So we have ot reset and cofigure it
		// Check if the code is running 
		if ( !CheckIfCardLoaded() ) {
			PAPIDeregisterCard(CARD_NUMBER);
			PAPICleanup();
			MessageBox(HWND_DESKTOP, "The card is not loaded - please run WINPLOAD to load the card.", "RxTester - Fatal Error", MB_ICONSTOP | MB_OK );
			return(0);
		}
		// Set the configuration
		if (!SetConfiguration() ) {
			PAPIDeregisterCard(CARD_NUMBER);
			PAPICleanup();
			MessageBox(HWND_DESKTOP, "Couldn't set the configuration. Try to reload the board.", "RxTester - Fatal Error", MB_ICONSTOP | MB_OK );
			return(0);
		}
		// Perform POS_SETUP. This sets up the filtering mode to be 0. It'll fail for 
		// NSR 1255 system, but still will be ok, so we don't check the return value.
		// Do it if you want.
		DoPosSetup();
	}
	//
	//------------ We're OK -------------

	// Create main dialog window
	exehwn = CreateDialog(ghinstance, "MAINWIN", HWND_DESKTOP, NULL);
	if (!exehwn) {
		PAPIDeregisterCard(CARD_NUMBER);
		PAPICleanup();
		MessageBox(HWND_DESKTOP, "Couldn't create main dialog window.", "RxTester - Fatal Error", MB_ICONSTOP | MB_OK );
		return(0);								// couldn't create it for some reason
	}
	SendMessage( exehwn, WM_INITDIALOG, 0, 0 );	// wanna set initial condition

	// Start the timer
	timerId = SetTimer( exehwn, 0,	POLL_TIMER,	(TIMERPROC)NULL );
	if (timerId == 0) {
		PAPIDeregisterCard(CARD_NUMBER);
		PAPICleanup();
		DestroyWindow( exehwn );
		MessageBox(HWND_DESKTOP, "Couldn't get system timer.", "RxTester - Fatal Error", MB_ICONSTOP | MB_OK );
		return(0);								// couldn't get a timer
	}

	fRetVal = TRUE;
	while (fRetVal)  {
		while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {

			if (msg.message == WM_QUIT)
				fRetVal = FALSE;

			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
	// Do all the cleanup
	PAPIDeregisterCard(CARD_NUMBER);		// deregister card
	PAPICleanup();							// cleanup the DLL
	KillTimer( 0, timerId );				// kill the timer
	return fRetVal;
}


//
// MainWndProc()
//
// Callback function for the main dialog window.
//
LRESULT CALLBACK MainWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam )
{
	char	outstr[100];
	UINT	i;

	switch(iMessage) {
	case WM_INITDIALOG:
		UpdateMe(hWnd);
		break;
	case WM_CLOSE:
		if (Receiving) { // if we're in receive mode ... disable POS
			DisablePos();
		}
		DestroyWindow(hWnd);
		PostQuitMessage(0);
		return(0);
	case WM_TIMER:
		if (Receiving) { // if we're in receive mode ...
			// Update poll box - check if unchecked, uncheck if checked
			CheckDlgButton( hWnd, IDC_POLL, !IsDlgButtonChecked( hWnd, IDC_POLL ));
			if (IsDataAvailable()) {
				SetDlgItemText(hWnd, IDC_DATASTATUS, "Data Available for Receiving");
				if (DoReceive()) { 
					// received some data - print it ...
					sprintf( outstr, "  %d\t %d\t", POSPort, cBlock.buffer_length );
					for( i=0; (i<20) && (i<cBlock.buffer_length); i++ ) {
						sprintf( outstr+strlen(outstr), "%02X ", cBlock.data[i] );
					}
					SayMessage( hWnd, outstr );	// put it into box ...
				}
			} else {
				SetDlgItemText(hWnd, IDC_DATASTATUS, "No Data Available for Receiving");
			}
		}
   		break;
	case WM_COMMAND:
		switch(GET_WM_COMMAND_ID(wParam, lParam)) {
		case IDC_PORT1:		// "Port 1" check box checked
			POSPort = 1;
			UpdateMe(hWnd);
			break;
		case IDC_PORT2:		// "Port 2" check box checked
			POSPort = 2;
			UpdateMe(hWnd);
			break;
		case IDC_START:		// "Start Receive" button clicked
			if ( EnablePos() ) {
				Receiving = TRUE;
				UpdateMe(hWnd);
			} else {
				MessageBox(hWnd, "Couldn't ENABLE_POS. Try to exit and reload the code.", "RxTester - Fatal Error", MB_ICONSTOP | MB_OK );
			}
			break;
		case IDC_STOP:		// "Stop Receive" button clicked
			if ( DisablePos() ) {
				Receiving = FALSE;
				UpdateMe(hWnd);
			} else {
				MessageBox(hWnd, "Couldn't DISABLE_POS. Try exit and reload the code.", "RxTester - Fatal Error", MB_ICONSTOP | MB_OK );
			}
			break;
		case IDC_QUIT:		// "Exit Tester" button clicked
			PostMessage(hWnd, WM_CLOSE, 0, 0L);
			break;
		}
		break;

	case WM_SYSCOMMAND:
		if (wParam == SC_CLOSE) {
			PostMessage(hWnd, WM_CLOSE, 0, 0L);
		}
		break;
	}
	return DefDlgProc( hWnd, iMessage, wParam, lParam );
}

//
// UpdateMe() - updates the states of controls of main dialog window
//
static void UpdateMe(HWND hWnd)
{
	// Update "Port 1" check box
	CheckDlgButton(hWnd, IDC_PORT1, (POSPort==1)?1:0 );
	// Update "Port 2" check box
	CheckDlgButton(hWnd, IDC_PORT2, (POSPort==2)?1:0 );
	
	// According to the value of Receiving - disablle some of controls
	if (Receiving) {
		EnableWindow(GetDlgItem(hWnd, IDC_PORT1), FALSE);	// disable "Port 1"
		EnableWindow(GetDlgItem(hWnd, IDC_PORT2), FALSE);	// disable "Port 2"
		EnableWindow(GetDlgItem(hWnd, IDC_START), FALSE);	// disable "Start ..."
		EnableWindow(GetDlgItem(hWnd, IDC_STOP), TRUE);		// enable "Stop ..."
		EnableWindow(GetDlgItem(hWnd, IDC_POLL), TRUE);		// enable "Checking for ..."
	} else {
		EnableWindow(GetDlgItem(hWnd, IDC_PORT1), TRUE);	// enable "Port 1"
		EnableWindow(GetDlgItem(hWnd, IDC_PORT2), TRUE);	// enable "Port 2"
		EnableWindow(GetDlgItem(hWnd, IDC_START), TRUE);	// enable "Start ..."
		EnableWindow(GetDlgItem(hWnd, IDC_STOP), FALSE);	// disable "Stop ..."
		EnableWindow(GetDlgItem(hWnd, IDC_POLL), FALSE);	// disable "Checking for ..."
		CheckDlgButton(hWnd, IDC_POLL, 0);
		SetDlgItemText(hWnd, IDC_DATASTATUS, "");
	}
	return;
}

// Put the message into the list box
static void SayMessage(HWND hWnd, char* message)
{
	static UINT		line_count = 0;		// number of lines inside the listbox
	const  UINT		Max_Lines = 100;	// maximum number of lines inside the listbox

	// delete first line if we reached the listbox limit
	if (line_count == Max_Lines ) {
		SendDlgItemMessage( hWnd, IDC_OUTPUT, LB_DELETESTRING, 0, 0 );
		line_count--;	
	}
	// put the line into listbox
	SendDlgItemMessage( hWnd, IDC_OUTPUT, LB_ADDSTRING, 0, (LPARAM)message );
	// select the last line
	SendDlgItemMessage( hWnd, IDC_OUTPUT, LB_SETCURSEL, line_count++, 0 );
	return;
}

//***********************************************************************************
// Real stuff starts here
//

//
// CheckIfCardLoaded() - Performs "warm reset" of the board - returns TRUE is the code
//						 seems to be loaded.
//
static BOOLEAN CheckIfCardLoaded(void)
{
	// Set up the Contol Block.
	cBlock.command			= RESET_CARD;
	cBlock.buffer_length	= 0;
	// Call the board - we return TRUE only if PAPIContol returned PAPI_STATUS_SUCCESS
	// and return code is RCODE_SUCCESS ( 0x00 )
	if ( (PAPIControl( &cBlock, CARD_NUMBER ) == PAPI_STATUS_SUCCESS) &&
		 (cBlock.return_code == RCODE_SUCCESS) ) {
		return TRUE;
	}
	return FALSE;
}

// 
// SetConfiguration() - sets the cofiguration of the board.
//
static BOOLEAN SetConfiguration(void)
{
	// Set up the Contol Block.
	cBlock.command				= CONFIGURE;
	cBlock.buffer_length		= 0x06;
	cBlock.data[0]				= 0x02;			// two POS ports
	*(PUSHORT)&(cBlock.data[1])	= 256;			// data field length
	cBlock.data[3]				= 0x01;			// number of async ports
	cBlock.data[4]				= LS_19200;		// line speed on async port 19200bps
	cBlock.data[5]				= MD_HALF_DPLX;	// async mode - half-duplx
	// Call the board - we return TRUE only if PAPIContol returned PAPI_STATUS_SUCCESS
	// and return code is RCODE_SUCCESS ( 0x00 )
	if ( (PAPIControl( &cBlock, CARD_NUMBER ) == PAPI_STATUS_SUCCESS) &&
		 (cBlock.return_code == RCODE_SUCCESS) ) {
		return TRUE;
	}
	return FALSE;
}

//
// EnablePos() - enables POS port and starting to listen
//
static BOOLEAN EnablePos(void)
{
	// Set up the Contol Block.
	cBlock.command			= ENABLE_POS;
	cBlock.buffer_length	= 0;
	cBlock.port_number		= POSPort;
	// Call the board - we return TRUE only if PAPIContol returned PAPI_STATUS_SUCCESS
	// and return code is RCODE_SUCCESS ( 0x00 )
	if ( (PAPIControl( &cBlock, CARD_NUMBER ) == PAPI_STATUS_SUCCESS) &&
		 (cBlock.return_code == RCODE_SUCCESS) ) {
		return TRUE;
	}
	return FALSE;
}

//
// DisablePos() - disables POS port
//
static BOOLEAN DisablePos(void)
{
	// Set up the Contol Block.
	cBlock.command			= DISABLE_POS;
	cBlock.buffer_length	= 0;
	cBlock.port_number		= POSPort;
	// Call the board - we return TRUE only if PAPIContol returned PAPI_STATUS_SUCCESS
	// and return code is RCODE_SUCCESS ( 0x00 )
	if ( (PAPIControl( &cBlock, CARD_NUMBER ) == PAPI_STATUS_SUCCESS) &&
		 (cBlock.return_code == RCODE_SUCCESS) ) {
		return TRUE;
	}
	return FALSE;
}

//
// IsDataAvailable() - Checks if there is data available for receving on the board
//					   It reads the card status byte - and checks if there is data
//					   for our port
//
static BOOLEAN IsDataAvailable(void)
{
	BYTE	tmp_byte;

	if ( PAPI_ERROR( PAPIReadByte( CARD_NUMBER, POS_STATE_OFF, &tmp_byte ))) {
		return FALSE;
	}
	if ((POSPort == 1) && (tmp_byte & POS_RX_DATA_AVAIL_1)) return TRUE;

	if ((POSPort == 2) && (tmp_byte & POS_RX_DATA_AVAIL_2)) return TRUE;
	return FALSE;
}

//
// DoReceive() - reads POS data
//
static BOOLEAN DoReceive(void)
{
	// Set up the Contol Block.
	cBlock.command			= RECEIVE_POS;
	cBlock.buffer_length	= 0;
	cBlock.port_number		= POSPort;
	// Call the board - we return TRUE only if PAPIContol returned PAPI_STATUS_SUCCESS
	// and return code is RCODE_SUCCESS ( 0x00 )
	if ( (PAPIControl( &cBlock, CARD_NUMBER ) == PAPI_STATUS_SUCCESS) &&
		 (cBlock.return_code == RCODE_SUCCESS) ) {
		return TRUE;
	}
	return FALSE;
}

//
// DoPosSetup() - performs POS_SETUP command. This will work correctly only for IBM4680
//				  and NCR 2126/2127.
//
static BOOLEAN DoPosSetup(void)
{
	// Set up the Contol Block.
	cBlock.command			= POS_SETUP;
	cBlock.buffer_length	= 0x03;
	cBlock.data[0]			= 0;			// no filtering
	// Call the board - we return TRUE only if PAPIContol returned PAPI_STATUS_SUCCESS
	// and return code is RCODE_SUCCESS ( 0x00 )
	if ( (PAPIControl( &cBlock, CARD_NUMBER ) == PAPI_STATUS_SUCCESS) &&
		 (cBlock.return_code == RCODE_SUCCESS) ) {
		return TRUE;
	}
	return FALSE;
}
